unit WinState;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TStateChange = procedure(Sender: TObject; State: TWindowState; LastState: TWindowState) of object;
  TWinState = class(TComponent)
  private
    { Private declarations }
    FState: uint;
    FLastState: uint;
    FTimer: TTimer;
    FInterval: integer;
    FHandle: hwnd;
    FActive: boolean;
    Finitilized: Boolean;
    FOnStateChange: TStateChange;
    FOnStateCheck: TNotifyEvent;
  protected
    { Protected declarations }
    constructor create(AOwner: TComponent); override;
    destructor Destroy; override;
    Procedure Loaded; Override;
    procedure GetStateInfo(sender: TObject);
    procedure SetActive(value: Boolean);
    procedure SetHandle(value: Hwnd);
    procedure SetInterval(value: integer);
  public
    { Public declarations }
  published
    { Published declarations }
    property OnStateChange: TStateChange read FOnStateChange write FOnStateChange;
    property OnStateCheck: TNotifyEvent read FOnStateCheck write FOnStateCheck;
    property WinHandle: hwnd read Fhandle write SetHandle;
    property Active: boolean read FActive write SetActive;
    property Interval: integer read FInterval write SetInterval;
  end;

procedure Register;

implementation

Procedure TWinState.Loaded;
begin
 inherited Loaded;
 FTimer.enabled := FActive;
end;

Procedure TWinState.SetHandle(value: Hwnd);
begin
     Finitilized := False;
     FHandle := Value;
end;

Procedure TWinState.SetActive(Value: Boolean);
begin
     Finitilized := False;
     FActive := Value;
     if value then GetStateInfo(Self);
     if not (csDesigning in ComponentState) then
        FTimer.Enabled := Value;
end;

Procedure TWinState.SetInterval(value: integer);
begin
     Ftimer.interval := value;
     FInterval := Value;
end;

Procedure TWinState.GetStateInfo(Sender: TObject);
var winplacement: PWindowPlacement;
    State: TWindowState;
    LastState: TWindowState;
begin
     if assigned(FOnStateCheck) then
        FOnStateCheck(self);
     if FHandle = 0 then exit;
     getmem(WinPlacement, sizeof(TWindowPlacement));
     WinPlacement.Length := sizeof(TWindowPlacement);
     if GetWindowPlacement(Fhandle, WinPlacement) then
     begin
          FState := winplacement.showcmd;
          if not Finitilized then
          begin
             FLastState := FState;
             Finitilized := True;
          end;
          if (FState <> FLastState) and
             (assigned(FOnStateChange)) then
             begin
                  case FLastState of
                    sw_minimize,
                    sw_showminimized:
                      LastState := wsminimized;
                    sw_shownormal: laststate := wsnormal;
                    sw_showmaximized: laststate := wsmaximized;
                  end;
                  case FState of
                    SW_MINIMIZE,
                    SW_SHOWMINIMIZED:
                    begin
                         FLastState := FState;
                         state := wsminimized;
                    end;
                    SW_SHOWNORMAL:
                    begin
                         FLastState := FState;
                         state := wsnormal;
                    end;
                    SW_SHOWMAXIMIZED:
                    begin
                         FLastState := FState;
                         State := wsmaximized;
                    end;
                  end;
                  FOnStateChange(Self, State, LastState);
             end;
     end;
     FreeMem(WinPlacement);
end;

constructor TWinState.create(AOwner: TComponent);
begin
     Inherited;
     FHandle := 0;
     FInterval := 500;
     Finitilized := False;
     FActive := False;
     FTimer := TTimer.create(self);
     FTimer.interval := FInterval;
     FTimer.OnTimer := GetStateInfo;
end;

Destructor TWinState.Destroy;
begin
     FTimer.enabled := False;
     FTimer.free;
     Inherited;
end;

procedure Register;
begin
  RegisterComponents('System', [TWinState]);
end;

end.
